@@ -115,7 +115,7 @@ static GScannerConfig scanner_config_template =
   FALSE                 /* symbol_2_token */,
   FALSE                 /* scope_0_fallback */,
 };
-static gchar		*std_marshaller_prefix = "g_cclosure_marshal";
+static gchar		* const std_marshaller_prefix = "g_cclosure_marshal";
 static gchar		*marshaller_prefix = "g_cclosure_user_marshal";
 static GHashTable	*marshallers = NULL;
 static gboolean		 gen_cheader = FALSE;
@@ -622,7 +622,7 @@ main (int   argc,
 
   /* add standard marshallers of the GObject library */
   if (std_includes)
-    for (i = 0; i < sizeof (gobject_marshallers) / sizeof (gobject_marshallers[0]); i++)
+    for (i = 0; i < G_N_ELEMENTS (gobject_marshallers); i++)
       {
 	gchar *tmp = g_strdup (gobject_marshallers[i]);
 	
@@ -652,7 +652,13 @@ main (int   argc,
   for (slist = files; slist; slist = slist->next)
     {
       gchar *file = slist->data;
-      gint fd = open (file, O_RDONLY);
+      gint fd;
+
+      if (strcmp (file, "/dev/stdin") == 0)
+	/* Mostly for Win32. This is equivalent to opening /dev/stdin */
+	fd = dup (0);
+      else
+	fd = open (file, O_RDONLY);
 
       if (fd < 0)
 	{
